import { onUnexpectedExternalError } from '../../../../base/common/errors.js';
class CodeActionKind {
    static { this.sep = '.'; }
    static { this.None = ( new CodeActionKind('@@none@@')); }
    static { this.Empty = ( new CodeActionKind('')); }
    static { this.QuickFix = ( new CodeActionKind('quickfix')); }
    static { this.Refactor = ( new CodeActionKind('refactor')); }
    static { this.RefactorExtract = CodeActionKind.Refactor.append('extract'); }
    static { this.RefactorInline = CodeActionKind.Refactor.append('inline'); }
    static { this.RefactorMove = CodeActionKind.Refactor.append('move'); }
    static { this.RefactorRewrite = CodeActionKind.Refactor.append('rewrite'); }
    static { this.Notebook = ( new CodeActionKind('notebook')); }
    static { this.Source = ( new CodeActionKind('source')); }
    static { this.SourceOrganizeImports = CodeActionKind.Source.append('organizeImports'); }
    static { this.SourceFixAll = CodeActionKind.Source.append('fixAll'); }
    static { this.SurroundWith = CodeActionKind.Refactor.append('surround'); }
    constructor(value) {
        this.value = value;
    }
    equals(other) {
        return this.value === other.value;
    }
    contains(other) {
        return this.equals(other) || this.value === '' || other.value.startsWith(this.value + CodeActionKind.sep);
    }
    intersects(other) {
        return this.contains(other) || other.contains(this);
    }
    append(part) {
        return ( new CodeActionKind(this.value + CodeActionKind.sep + part));
    }
}
var CodeActionTriggerSource;
( (function(CodeActionTriggerSource) {
    CodeActionTriggerSource["Refactor"] = "refactor";
    CodeActionTriggerSource["RefactorPreview"] = "refactor preview";
    CodeActionTriggerSource["Lightbulb"] = "lightbulb";
    CodeActionTriggerSource["Default"] = "other (default)";
    CodeActionTriggerSource["SourceAction"] = "source action";
    CodeActionTriggerSource["QuickFix"] = "quick fix action";
    CodeActionTriggerSource["FixAll"] = "fix all";
    CodeActionTriggerSource["OrganizeImports"] = "organize imports";
    CodeActionTriggerSource["AutoFix"] = "auto fix";
    CodeActionTriggerSource["QuickFixHover"] = "quick fix hover window";
    CodeActionTriggerSource["OnSave"] = "save participants";
    CodeActionTriggerSource["ProblemsView"] = "problems view";
})(CodeActionTriggerSource || (CodeActionTriggerSource = {})));
function mayIncludeActionsOfKind(filter, providedKind) {
    if (filter.include && !filter.include.intersects(providedKind)) {
        return false;
    }
    if (filter.excludes) {
        if (( filter.excludes.some(exclude => excludesAction(providedKind, exclude, filter.include)))) {
            return false;
        }
    }
    if (!filter.includeSourceActions && CodeActionKind.Source.contains(providedKind)) {
        return false;
    }
    return true;
}
function filtersAction(filter, action) {
    const actionKind = action.kind ? ( new CodeActionKind(action.kind)) : undefined;
    if (filter.include) {
        if (!actionKind || !filter.include.contains(actionKind)) {
            return false;
        }
    }
    if (filter.excludes) {
        if (actionKind && ( filter.excludes.some(exclude => excludesAction(actionKind, exclude, filter.include)))) {
            return false;
        }
    }
    if (!filter.includeSourceActions) {
        if (actionKind && CodeActionKind.Source.contains(actionKind)) {
            return false;
        }
    }
    if (filter.onlyIncludePreferredActions) {
        if (!action.isPreferred) {
            return false;
        }
    }
    return true;
}
function excludesAction(providedKind, exclude, include) {
    if (!exclude.contains(providedKind)) {
        return false;
    }
    if (include && exclude.contains(include)) {
        return false;
    }
    return true;
}
class CodeActionCommandArgs {
    static fromUser(arg, defaults) {
        if (!arg || typeof arg !== 'object') {
            return ( new CodeActionCommandArgs(defaults.kind, defaults.apply, false));
        }
        return ( new CodeActionCommandArgs(
            CodeActionCommandArgs.getKindFromUser(arg, defaults.kind),
            CodeActionCommandArgs.getApplyFromUser(arg, defaults.apply),
            CodeActionCommandArgs.getPreferredUser(arg)
        ));
    }
    static getApplyFromUser(arg, defaultAutoApply) {
        switch (typeof arg.apply === 'string' ? arg.apply.toLowerCase() : '') {
            case 'first': return "first" ;
            case 'never': return "never" ;
            case 'ifsingle': return "ifSingle" ;
            default: return defaultAutoApply;
        }
    }
    static getKindFromUser(arg, defaultKind) {
        return typeof arg.kind === 'string'
            ? ( new CodeActionKind(arg.kind))
            : defaultKind;
    }
    static getPreferredUser(arg) {
        return typeof arg.preferred === 'boolean'
            ? arg.preferred
            : false;
    }
    constructor(kind, apply, preferred) {
        this.kind = kind;
        this.apply = apply;
        this.preferred = preferred;
    }
}
class CodeActionItem {
    constructor(action, provider, highlightRange) {
        this.action = action;
        this.provider = provider;
        this.highlightRange = highlightRange;
    }
    async resolve(token) {
        if (this.provider?.resolveCodeAction && !this.action.edit) {
            let action;
            try {
                action = await this.provider.resolveCodeAction(this.action, token);
            }
            catch (err) {
                onUnexpectedExternalError(err);
            }
            if (action) {
                this.action.edit = action.edit;
            }
        }
        return this;
    }
}
export { CodeActionCommandArgs, CodeActionItem, CodeActionKind, CodeActionTriggerSource, filtersAction, mayIncludeActionsOfKind };
